@@ -94,8 +94,8 @@ pub struct FlatpakModule {
     pub cmake: Option<bool>,
 
     /// Build system to use.
-    #[serde(deserialize_with = "crate::build_system::deserialize_from_string")]
-    #[serde(serialize_with = "crate::build_system::serialize_to_string")]
+    #[serde(deserialize_with = "crate::build_system::FlatpakBuildSystem::deserialize")]
+    #[serde(serialize_with = "crate::build_system::FlatpakBuildSystem::serialize")]
     #[serde(skip_serializing_if = "Option::is_none")]
     pub buildsystem: Option<FlatpakBuildSystem>,
 
@@ -248,21 +248,10 @@ impl FlatpakModule {
     }
 
     pub fn parse(format: FlatpakManifestFormat, manifest_content: &str) -> Result<FlatpakModule, String> {
-        let mut flatpak_module: FlatpakModule = match &format {
-            FlatpakManifestFormat::YAML => match serde_yaml::from_str(&manifest_content) {
-                Ok(m) => m,
-                Err(e) => {
-                    return Err(format!("Failed to parse the Flatpak module manifest: {}.", e));
-                }
-            },
-            FlatpakManifestFormat::JSON => {
-                let json_content_without_comments = crate::utils::remove_comments_from_json(manifest_content);
-                match serde_json::from_str(&json_content_without_comments) {
-                    Ok(m) => m,
-                    Err(e) => {
-                        return Err(format!("Failed to parse the Flatpak module manifest: {}.", e));
-                    }
-                }
+        let mut flatpak_module: FlatpakModule = match format.parse(manifest_content) {
+            Ok(m) => m,
+            Err(e) => {
+                return Err(format!("Failed to parse the Flatpak module manifest: {}.", e));
             }
         };
         flatpak_module.format = format;
@@ -284,15 +273,24 @@ impl FlatpakModule {
                 return Err("Sources provided as strings cannot be URLs!".to_string());
             }
         }
+        for source in &flatpak_module.sources {
+            let source_description = match source {
+                FlatpakSourceItem::Path(_) => continue,
+                FlatpakSourceItem::Description(d) => d,
+            };
+            if let Err(e) = source_description.is_valid() {
+                return Err(e);
+            }
+        }
 
         Ok(flatpak_module)
     }
 
     pub fn dump(&self) -> Result<String, String> {
-        return match serde_yaml::to_string(&self) {
+        match self.format.dump(self) {
             Ok(d) => Ok(d),
-            Err(e) => Err(format!("Failed to dump the Flatpak manifest: {}.", e)),
-        };
+            Err(e) => return Err(format!("Failed to dump the Flatpak manifest: {}.", e)),
+        }
     }
 
     pub fn file_path_matches(path: &str) -> bool {
@@ -601,6 +599,30 @@ mod tests {
         }
     }
 
+    #[test]
+    #[ignore]
+    pub fn test_parse_builddir() {
+        // FIXME why is this failing?
+        // yes should be a valid boolean value, no?
+        let module_manifest = r###"
+            name: fmt
+            buildsystem: cmake-ninja
+            builddir: yes
+            config-opts:
+              - "-DFMT_TEST=OFF"
+            sources:
+              - type: git
+                url: https://github.com/fmtlib/fmt.git
+                commit: 561834650aa77ba37b15f7e5c9d5726be5127df9
+        "###;
+        match FlatpakModule::parse(FlatpakManifestFormat::YAML, module_manifest) {
+            Err(e) => std::panic::panic_any(e),
+            Ok(module) => {
+                assert_eq!(module.name, "fmt");
+            }
+        }
+    }
+
     #[test]
     pub fn test_parse_extra_data() {
         let module_manifest = r###"
@@ -654,6 +676,19 @@ mod tests {
         assert!(FlatpakModule::parse(FlatpakManifestFormat::YAML, helm_file,).is_err())
     }
 
+    #[test]
+    pub fn test_parse_invalid_source() {
+        let file = r###"
+            name: wps
+            sources:
+              - path: "^empty\\.c$"
+                isGenerated: false
+                sourceGroupName: "Source Files",
+                compileGroupLanguage: C
+        "###;
+        assert!(FlatpakModule::parse(FlatpakManifestFormat::YAML, file).is_err())
+    }
+
     #[test]
     pub fn test_parse_no_buildsystem() {
         let module_manifest = r###"
